@@ -1,3 +1,4 @@
+if(FALSE) {
 .First.lib <-
 function(libname, pkgname)
 {
@@ -10,14 +11,19 @@ function(libname, pkgname)
    # Set the error handlers to our local ones.
  .C("RSXML_setErrorHandlers", PACKAGE = "XML")
 }
+}
 
 .onLoad =
 function(libname, pkgname)
 {
+
   # Added by Uwe Ligges.
  if(.Platform$OS.type == "windows"){
+
+     fixPath = base::normalizePath
+
      temp <- Sys.getenv("PATH")
-     Sys.setenv("PATH" = paste(utils::normalizePath(file.path(libname, pkgname, "libs")), 
+     Sys.setenv("PATH" = paste(fixPath(file.path(libname, pkgname, "libs")), 
                                file.path(Sys.getenv("R_HOME"), "modules", fsep="\\"), temp, sep=";"))
      on.exit(Sys.setenv(PATH = temp))
  }     
