@@ -32,7 +32,7 @@ RS_XML(invokeFunction)(USER_OBJECT_ fun, USER_OBJECT_ opArgs, USER_OBJECT_ data,
   USER_OBJECT_ ans;
   int addContext = 0;
 
-  if(context && TYPEOF(fun) == CLOSXP && OBJECT(fun) && R_isInstanceOf(fun, XML_PARSE_CONTEXT_FUNCTION))
+  if(context && TYPEOF(fun) == CLOSXP && isObject(fun) && R_isInstanceOf(fun, XML_PARSE_CONTEXT_FUNCTION))
       addContext = 1;
 
   n = Rf_length(opArgs) + addContext + 1;
@@ -286,7 +286,7 @@ R_isInstanceOf(USER_OBJECT_ obj, const char *klass)
 
 
 SEXP
-RS_XML_getStructuredErrorHandler()
+RS_XML_getStructuredErrorHandler(void)
 {
     SEXP ans;
     PROTECT(ans = NEW_LIST(2));
@@ -307,8 +307,7 @@ RS_XML_setStructuredErrorHandler(SEXP els)
     sym = VECTOR_ELT(els, 1);
 
     if(sym != R_NilValue && TYPEOF(sym) != EXTPTRSXP) {
-	PROBLEM "invalid symbol object for XML error handler. Need an external pointer, e.g from getNativeSymbolInfo"
-        ERROR;
+	Rf_error("invalid symbol object for XML error handler. Need an external pointer, e.g from getNativeSymbolInfo");
     }
 
     if(fun == R_NilValue)
