@@ -87,7 +87,7 @@ R_newXMLTextNode(USER_OBJECT_ value, USER_OBJECT_ sdoc, SEXP manageMemory)
 
     txt = CHAR_TO_XMLCHAR(CHAR_DEREF(STRING_ELT(value, 0)));
     if(doc)
-	node = xmlNewDocTextLen(doc, txt, strlen(XMLCHAR_TO_CHAR(txt)));
+	node = xmlNewDocTextLen(doc, txt, (int)strlen(XMLCHAR_TO_CHAR(txt)));
     else
 	node = xmlNewText(txt);
 	
@@ -106,7 +106,7 @@ R_newXMLCDataNode(USER_OBJECT_ sdoc, USER_OBJECT_ value, USER_OBJECT_ manageMemo
 
   tmp = CHAR_DEREF(STRING_ELT(value,0));
 
-  node = xmlNewCDataBlock(doc, CHAR_TO_XMLCHAR(tmp), strlen(tmp));
+  node = xmlNewCDataBlock(doc, CHAR_TO_XMLCHAR(tmp), (int)strlen(tmp));
 
   return(R_createXMLNodeRef(node, manageMemory));
 }
@@ -221,7 +221,7 @@ removeNodeNamespaceByName(xmlNodePtr node, const char * const id)
 	return(0);
     prev = node->nsDef;
     p = node->nsDef;
-    if(!(id[0] && !p->prefix) || (p->prefix && strcmp(p->prefix, id) ==  0)) {
+    if(!(id[0] && !p->prefix) || (p->prefix && strcmp((const char *)p->prefix, id) ==  0)) {
                 /*XXX Free or not */
         if(node->ns == p)
 	    node->ns = NULL;
@@ -230,7 +230,7 @@ removeNodeNamespaceByName(xmlNodePtr node, const char * const id)
     }
 
     while(1) {
-	if((!id[0] && !p->prefix) || (p->prefix && strcmp(p->prefix, id) == 0)) {
+	if((!id[0] && !p->prefix) || (p->prefix && strcmp((const char *)p->prefix, id) == 0)) {
 	    prev->next = p->next;
 	    if(node->ns == p)
 		node->ns = NULL;
@@ -248,8 +248,7 @@ R_replaceDummyNS(USER_OBJECT_ s_node, USER_OBJECT_ newNS, USER_OBJECT_ prefix)
 {
     xmlNodePtr node;
     if(TYPEOF(s_node) != EXTPTRSXP) {
-	PROBLEM "non external pointer passed to R_replaceDummyNS"
-	    ERROR;
+	Rf_error("non external pointer passed to R_replaceDummyNS");
     }
 	
     node = (xmlNodePtr) R_ExternalPtrAddr(s_node);
@@ -305,7 +304,7 @@ RS_XML_removeNodeNamespaces(SEXP s_node, SEXP r_ns)
 	    LOGICAL(ans)[i] = removeNodeNamespaceByName(node, prefix);
 	} else if(TYPEOF(el) == EXTPTRSXP) {
 	    xmlNsPtr p = (xmlNsPtr) R_ExternalPtrAddr(el);
-	    LOGICAL(ans)[i] = removeNodeNamespaceByName(node, p->prefix);
+	    LOGICAL(ans)[i] = removeNodeNamespaceByName(node, (const char *)p->prefix);
 	}
     }
 
@@ -348,7 +347,9 @@ RS_XML_removeNodeAttributes(USER_OBJECT_ s_node, USER_OBJECT_ attrs, USER_OBJECT
 	    xmlNsPtr ns = NULL;
 	    xmlChar *id;
 	    id = CHAR_TO_XMLCHAR(CHAR_DEREF(STRING_ELT(attr_names, i)));
-	    ns = (xmlNsPtr) R_ExternalPtrAddr(VECTOR_ELT(attrs, i));
+	    SEXP ai = VECTOR_ELT(attrs, i);
+	    if(TYPEOF(ai) == EXTPTRSXP)
+		ns = (xmlNsPtr) R_ExternalPtrAddr(ai);
 	    if(id[0])
 		INTEGER(ans)[i] = xmlUnsetNsProp(node, ns, id);
 	} else
@@ -417,14 +418,12 @@ R_removeInternalNode(SEXP r_node, SEXP r_free)
     for(i = 0; i < n; i++) {
 	SEXP el = VECTOR_ELT(r_node, i);
 	if(TYPEOF(el) != EXTPTRSXP) {
-	    PROBLEM "removeInternalNode needs ans external pointer object"
-	    ERROR;
+	    Rf_error("removeInternalNode needs ans external pointer object");
 	}
 
 	node = (xmlNodePtr) R_ExternalPtrAddr(el);
 	if(!node) {
-	    PROBLEM "removeInternalNode ignoring a NULL external pointer object"
-		WARN;	
+	    Rf_warning("removeInternalNode ignoring a NULL external pointer object");	
 	}
 	xmlUnlinkNode(node);
     
@@ -480,11 +479,11 @@ R_isNodeChildOfAt(SEXP rkid, SEXP rnode, SEXP rat)
 USER_OBJECT_
 R_insertXMLNode(USER_OBJECT_ node, USER_OBJECT_ parent, USER_OBJECT_ at, USER_OBJECT_ shallow)
 {
-    xmlNodePtr n, p, check, tmp = NULL;
+    // check is currently set but unused.
+    xmlNodePtr n, p, /*check,*/ tmp = NULL;
     
     if(TYPEOF(parent) != EXTPTRSXP) {
-       PROBLEM "R_insertXMLNode expects XMLInternalNode objects for the parent node"
-       ERROR;
+	Rf_error("R_insertXMLNode expects XMLInternalNode objects for the parent node");
     }
 
     if(IS_LIST(node))  {
@@ -500,23 +499,21 @@ R_insertXMLNode(USER_OBJECT_ node, USER_OBJECT_ parent, USER_OBJECT_ at, USER_OB
         int i;
 	p = (xmlNodePtr) R_ExternalPtrAddr(parent);	
 	for(i = 0; i < GET_LENGTH(node); i++) {
-  	    n = xmlNewText(CHAR(STRING_ELT(node, i)));
+  	    n = xmlNewText((const xmlChar *)CHAR(STRING_ELT(node, i)));
    	    xmlAddChild(p, n);
 	}
 	return(NULL_USER_OBJECT);
     }
 
     if(TYPEOF(node) != EXTPTRSXP) {
-       PROBLEM "R_insertXMLNode expects XMLInternalNode objects"
-       ERROR;
+	Rf_error("R_insertXMLNode expects XMLInternalNode objects");
     }
 
     p = (xmlNodePtr) R_ExternalPtrAddr(parent);
     n = (xmlNodePtr) R_ExternalPtrAddr(node);
 
     if(!p || !n) {
-	PROBLEM "either the parent or child node is NULL"
-        ERROR;
+	Rf_error("either the parent or child node is NULL");
     }
 
 #if 0
@@ -556,7 +553,7 @@ R_insertXMLNode(USER_OBJECT_ node, USER_OBJECT_ parent, USER_OBJECT_ at, USER_OB
  	           incrementDocRefBy(p->doc, getNodeCount(n));
             }
 	}
-	check = xmlAddChild(p, tmp);
+	/* check = */ xmlAddChild(p, tmp);
 
 #if 0
 /* XXXX */
@@ -566,7 +563,7 @@ R_insertXMLNode(USER_OBJECT_ node, USER_OBJECT_ parent, USER_OBJECT_ at, USER_OB
 	break;
     case XML_DOCUMENT_NODE:	
     case XML_HTML_DOCUMENT_NODE:	
-	check = xmlAddChild(p, n);
+	/*check = */ xmlAddChild(p, n);
 	incrementDocRef((xmlDocPtr) p);
 	break;
     case XML_PI_NODE:
@@ -574,9 +571,8 @@ R_insertXMLNode(USER_OBJECT_ node, USER_OBJECT_ parent, USER_OBJECT_ at, USER_OB
 	break;
     default:
        {
-	   PROBLEM "ignoring request to add child (types parent: %d, child %d)",
-               p->type, n->type
-	    WARN
+	   Rf_warning("ignoring request to add child (types parent: %d, child %d)",
+		      p->type, n->type);
        }
 	break;
     }
@@ -604,21 +600,18 @@ RS_XML_xmlAddSiblingAt(USER_OBJECT_ r_to, USER_OBJECT_ r_node, USER_OBJECT_ r_af
     xmlNodePtr (*f)(xmlNodePtr, xmlNodePtr);
 
     if(TYPEOF(r_to) != EXTPTRSXP) {
-       PROBLEM "RS_XML_xmlAddSiblingAt expects XMLInternalNode objects for the parent node"
-       ERROR;
+	Rf_error("RS_XML_xmlAddSiblingAt expects XMLInternalNode objects for the parent node");
     }
 
     if(TYPEOF(r_node) != EXTPTRSXP) {
-       PROBLEM "RS_XML_xmlAddSiblingAt expects XMLInternalNode objects for the node to add"
-       ERROR;
+	Rf_error("RS_XML_xmlAddSiblingAt expects XMLInternalNode objects for the node to add");
     }
 
     p = (xmlNodePtr) R_ExternalPtrAddr(r_to);
     n = (xmlNodePtr) R_ExternalPtrAddr(r_node);
 
     if(!p || !n) {
-	PROBLEM "either the parent or child node is NULL"
-        ERROR;
+	Rf_error("either the parent or child node is NULL");
     }
 	
     f = LOGICAL(r_after)[0] ?  xmlAddNextSibling : xmlAddPrevSibling ;
@@ -639,15 +632,13 @@ RS_XML_replaceXMLNode(USER_OBJECT_ r_old, USER_OBJECT_ r_new, USER_OBJECT_ manag
     xmlNodePtr Old, New, ans;
 
     if(TYPEOF(r_old) != EXTPTRSXP && TYPEOF(r_new) != EXTPTRSXP) {
-       PROBLEM "R_replaceXMLNode expects XMLInternalNode objects"
-       ERROR;
+	Rf_error("R_replaceXMLNode expects XMLInternalNode objects");
     }
     Old = (xmlNodePtr) R_ExternalPtrAddr(r_old);
     New = (xmlNodePtr) R_ExternalPtrAddr(r_new);
 
     if(!Old) {
-	PROBLEM "NULL value for XML node to replace"
-	    ERROR;
+	Rf_error("NULL value for XML node to replace");
     }
 
     ans = xmlReplaceNode(Old, New);
@@ -669,8 +660,7 @@ RS_XML_removeChildren(USER_OBJECT_ s_node, USER_OBJECT_ kids, USER_OBJECT_ freeN
 	node = (xmlNodePtr) R_ExternalPtrAddr(s_node);
     
 	if(!node) {
-	    PROBLEM "Empty XMLInternalNode"
-		ERROR;
+	    Rf_error("Empty XMLInternalNode");
 	}
     }
     
@@ -681,8 +671,7 @@ RS_XML_removeChildren(USER_OBJECT_ s_node, USER_OBJECT_ kids, USER_OBJECT_ freeN
 	if(!tmp)
 	    continue;
 	if(node && tmp->parent != node) {
-	    PROBLEM "trying to remove a child node from a different parent node"
-   	     ERROR;
+	    Rf_error("trying to remove a child node from a different parent node");
 	}
 
 	xmlUnlinkNode(tmp);
@@ -705,8 +694,7 @@ R_xmlRootNode(USER_OBJECT_ sdoc, USER_OBJECT_ skipDtd, USER_OBJECT_ manageMemory
       node = doc->children;
   
   if(!node) {
-      PROBLEM "empty XML document"
-	  WARN;
+      Rf_warning("empty XML document");
       return(NULL_USER_OBJECT);
   }
 
@@ -740,7 +728,7 @@ R_newXMLDoc(USER_OBJECT_ dtd, USER_OBJECT_ namespaces, USER_OBJECT_ isHTML)
       const char *d = (TYPEOF(dtd) == STRSXP && Rf_length(dtd)) ? 
                                 CHAR_DEREF(STRING_ELT(dtd, 0)) : NULL;
       if(d[0] == '5')
-	  doc = htmlNewDoc("", NULL);
+	  doc = htmlNewDoc((const xmlChar *)"", NULL);
       else
 	  doc = htmlNewDocNoDtD(d && d[0] ? CHAR_TO_XMLCHAR(d) : NULL, NULL);
 
@@ -856,13 +844,11 @@ USER_OBJECT_
 RS_XML_clone(USER_OBJECT_ obj, USER_OBJECT_ recursive, USER_OBJECT_ manageMemory)
 {
     if(TYPEOF(obj) != EXTPTRSXP) {
-	PROBLEM  "clone can only be applied to an internal, C-level libxml2 object"
-        ERROR;
+	Rf_error( "clone can only be applied to an internal, C-level libxml2 object");
     }
 
     if(!R_ExternalPtrAddr(obj)) {
-	PROBLEM  "NULL value passed to clone, possibly from a previous session"
-        ERROR;
+	Rf_error( "NULL value passed to clone, possibly from a previous session");
     }
 
     if(R_isInstanceOf(obj, "XMLInternalElementNode")) {
@@ -876,8 +862,7 @@ RS_XML_clone(USER_OBJECT_ obj, USER_OBJECT_ recursive, USER_OBJECT_ manageMemory
 	return(R_createXMLDocRef(xmlCopyDoc(doc, INTEGER(recursive)[0]))); // , manageMemory));
     }
     
-    PROBLEM "clone doesn't (yet) understand this internal data type"
-    ERROR;
+    Rf_error("clone doesn't (yet) understand this internal data type");
 
     return(NULL_USER_OBJECT); /* never reached */
 }
@@ -920,8 +905,7 @@ USER_OBJECT_
 R_removeXMLNsRef(xmlNsPtr ns)
 {
 /*XXX    xmlNsPtr p = (xmlNsPtr) R_ExternalPtrAddr(); */
-    PROBLEM "C routine R_removeXMLNsRef() not implemented yet" 
-    ERROR;
+    Rf_error("C routine R_removeXMLNsRef() not implemented yet");
     return(R_NilValue);
 }
 
@@ -941,17 +925,16 @@ R_createXMLNsRef(xmlNsPtr ns)
 USER_OBJECT_
 R_convertXMLNsRef(SEXP r_ns)
 {
-  SEXP ref, ans;
+  SEXP ans;
   xmlNsPtr ns;
 
   if(TYPEOF(r_ns) != EXTPTRSXP) {
-      PROBLEM "wrong type for namespace reference"
-	  ERROR;
+      Rf_error("wrong type for namespace reference");
   }
 
   ns = (xmlNsPtr) R_ExternalPtrAddr(r_ns);
 
-  PROTECT(ans =  mkString(ns->href));
+  PROTECT(ans =  mkString((const char *)ns->href));
   SET_NAMES(ans, mkString(ns->prefix ? XMLCHAR_TO_CHAR(ns->prefix) : ""));
 
   UNPROTECT(1);
@@ -1095,7 +1078,7 @@ R_createXMLNodeRef(xmlNodePtr node, USER_OBJECT_ finalize)
   addFinalizer = R_XML_getManageMemory(finalize, node->doc, node);
 
 /*  !IS_NOT_OUR_NODE_TO_TOUCH(node) */
-  if(addFinalizer && ((node->_private && ((int*)node->_private)[1] == R_MEMORY_MANAGER_MARKER)
+  if(addFinalizer && ((node->_private && ((int*)node->_private)[1] == (int) R_MEMORY_MANAGER_MARKER)
 		      || !node->doc || (!(IS_NOT_OUR_DOC_TO_TOUCH(node->doc))))) {
       if(node->_private == NULL) {
         node->_private = calloc(2, sizeof(int));
@@ -1125,7 +1108,7 @@ SEXP
 R_addXMLNodeFinalizer(SEXP r_node)
 {
 #ifdef XML_REF_COUNT_NODES /* ??? should this be ifndef or ifdef.??  */
-   xmlNodePtr node = (xmlNodePtr) R_ExternalPtrAddr(r_node);
+//   xmlNodePtr node = (xmlNodePtr) R_ExternalPtrAddr(r_node);
   R_RegisterCFinalizer(r_node, decrementNodeRefCount);
 #endif
   return(r_node);
@@ -1165,8 +1148,7 @@ R_saveXMLDOM(USER_OBJECT_ sdoc, USER_OBJECT_ sfileName, USER_OBJECT_ compression
     const char *encoding = CHAR_DEREF(STRING_ELT(r_encoding, 0));
     
     if(TYPEOF(sdoc) != EXTPTRSXP) {
-       PROBLEM "document passed to R_saveXMLDOM is not an external pointer"
-	   ERROR;
+	Rf_error("document passed to R_saveXMLDOM is not an external pointer");
     }
 
     doc = (xmlDocPtr) R_ExternalPtrAddr(sdoc);
@@ -1205,7 +1187,9 @@ R_saveXMLDOM(USER_OBJECT_ sdoc, USER_OBJECT_ sfileName, USER_OBJECT_ compression
 	    xmlSetDocCompressMode(doc, INTEGER_DATA(compression)[0]);
 	}
 	if(encoding && encoding[0])
-	    xmlSaveFileEnc(CHAR_DEREF(STRING_ELT(sfileName, 0)),  doc, encoding);
+// xmlSaveFileEnc doesn't indent. So use xmlSaveFormatFileEnc(). Issue identified by Earl Brown.
+//	    xmlSaveFileEnc(CHAR_DEREF(STRING_ELT(sfileName, 0)),  doc, encoding);
+	    xmlSaveFormatFileEnc(CHAR_DEREF(STRING_ELT(sfileName, 0)),  doc, encoding, LOGICAL_DATA(sindent)[0]);
 #if 0
 	else
 	    xmlSaveFile(CHAR_DEREF(STRING_ELT(sfileName, 0)),  doc);
@@ -1213,6 +1197,9 @@ R_saveXMLDOM(USER_OBJECT_ sdoc, USER_OBJECT_ sfileName, USER_OBJECT_ compression
 	else {
 	  FILE *f;
 	  f = fopen(CHAR_DEREF(STRING_ELT(sfileName, 0)), "w");
+	  if(!f) {
+	      Rf_error("cannot create file %s. Check the directory exists and permissions are appropriate", CHAR_DEREF(STRING_ELT(sfileName, 0)) );
+          }
 	  xmlDocFormatDump(f, doc, 1);
 	  fclose(f);
 	}
@@ -1249,8 +1236,7 @@ R_saveXMLDOM(USER_OBJECT_ sdoc, USER_OBJECT_ sfileName, USER_OBJECT_ compression
 	    xmlFree(mem);
 	} else { 
                /*XXX get the error message from libxml2 */
-	    PROBLEM "failed to write XML document contents"
-		ERROR;
+	    Rf_error("failed to write XML document contents");
 	}
         UNPROTECT(1);
 
@@ -1453,9 +1439,9 @@ RS_XML_printXMLNode(USER_OBJECT_ r_node, USER_OBJECT_ level, USER_OBJECT_ format
     if(xbuf->use > 0) {
         /*XXX this const char * in CHARSXP means we have to make multiple copies. */
      if(INTEGER(r_encoding_int)[0] == CE_NATIVE)
-        ans = ScalarString(CreateCharSexpWithEncoding(encoding, xbuf->content));
+        ans = ScalarString(CreateCharSexpWithEncoding((const xmlChar *)encoding, (const xmlChar *)xbuf->content));
      else
-        ans = ScalarString(mkCharCE(xbuf->content, INTEGER(r_encoding_int)[0]));
+        ans = ScalarString(mkCharCE((const char *)xbuf->content, INTEGER(r_encoding_int)[0]));
     } else
       ans = NEW_CHARACTER(1);
 
@@ -1469,8 +1455,7 @@ R_setXMLInternalTextNode_noenc(SEXP node)
 {
      xmlNodePtr n = (xmlNodePtr) R_ExternalPtrAddr(node);
      if(!n) {
-	 PROBLEM "null value passed for XMLInternalTextNode"
-	     ERROR;
+	 Rf_error("null value passed for XMLInternalTextNode");
      }
      n->name = (const xmlChar *) (&xmlStringTextNoenc);
      return(ScalarLogical(TRUE));
@@ -1484,15 +1469,14 @@ R_setXMLInternalTextNode_value(SEXP node, SEXP value)
 //   xmlChar *tmp;
    const char *str;
 
-   DECL_ENCODING_FROM_NODE(n)
+   // DECL_ENCODING_FROM_NODE(n)
 
    if(n->type != XML_TEXT_NODE) {
-       PROBLEM  "Can only set value on an text node"
-	   ERROR;
+       Rf_error( "Can only set value on an text node");
    }
 
    str = CHAR(STRING_ELT(value, 0));
-   xmlNodeSetContent(n, str);
+   xmlNodeSetContent(n, (const xmlChar *)str);
     
    return(node);
 }
@@ -1514,8 +1498,7 @@ R_xmlNodeValue(SEXP node, SEXP raw, SEXP r_encoding)
    DECL_ENCODING_FROM_NODE(n)
 
    if(!n) {
-      PROBLEM  "null value for xml node reference"
-      ERROR;
+       Rf_error( "null value for xml node reference");
    }
 
    tmp  = xmlNodeGetContent(n);
@@ -1533,7 +1516,7 @@ R_xmlNodeValue(SEXP node, SEXP raw, SEXP r_encoding)
      if(INTEGER(r_encoding)[0] == CE_NATIVE)
         ans = ScalarString(CreateCharSexpWithEncoding(encoding, tmp));
      else
-        ans = ScalarString(mkCharCE(tmp, INTEGER(r_encoding)[0]));
+        ans = ScalarString(mkCharCE((const char *)tmp, INTEGER(r_encoding)[0]));
 
 
      free(tmp);
@@ -1592,8 +1575,7 @@ RS_XML_isDescendantOf(USER_OBJECT_ r_node, USER_OBJECT_ r_top, USER_OBJECT_ stri
     top = (xmlNodePtr) R_ExternalPtrAddr(r_top);    
 
     if(!node || !top) {
-	PROBLEM  "null value passed to RS_XML_isDescendantOf"
-	    ERROR;
+	Rf_error( "null value passed to RS_XML_isDescendantOf");
     }
 
 /*XXX */
@@ -1689,16 +1671,16 @@ R_xmlNode_removeFinalizers(SEXP r_node)
 SEXP
 R_xmlSearchNs(SEXP r_doc, SEXP r_node, SEXP r_ns, SEXP r_asPrefix)
 {
-    const char * val;
+    const xmlChar * val;
     xmlNsPtr ns;
 
-    xmlDocPtr doc = (xmlDocPtr) r_doc == NULL_USER_OBJECT ? NULL : R_ExternalPtrAddr(r_doc);    
+    xmlDocPtr doc = (r_doc == NULL_USER_OBJECT) ? NULL : R_ExternalPtrAddr(r_doc);    
     xmlNodePtr node = (xmlNodePtr) R_ExternalPtrAddr(r_node);    
 
     if(Rf_length(r_ns) == 0)
 	return(NEW_CHARACTER(0));
 
-    val = CHAR_DEREF(STRING_ELT(r_ns, 0));
+    val = (const xmlChar *)CHAR_DEREF(STRING_ELT(r_ns, 0));
     
     ns = LOGICAL(r_asPrefix)[0] ? xmlSearchNs(doc, node, val) : xmlSearchNsByHref(doc, node, val);
 
@@ -1706,7 +1688,7 @@ R_xmlSearchNs(SEXP r_doc, SEXP r_node, SEXP r_ns, SEXP r_asPrefix)
 	return(NEW_CHARACTER(0));
     else {
 	SEXP r_ans;
-	PROTECT(r_ans =  mkString(ns->href));
+	PROTECT(r_ans =  mkString((const char *)ns->href));
 	SET_NAMES(r_ans, mkString(ns->prefix ? XMLCHAR_TO_CHAR(ns->prefix) : ""));
 	UNPROTECT(1);
 	return(r_ans);
@@ -1717,7 +1699,7 @@ R_xmlSearchNs(SEXP r_doc, SEXP r_node, SEXP r_ns, SEXP r_asPrefix)
 USER_OBJECT_
 R_getChildByIndex(USER_OBJECT_ r_node, USER_OBJECT_ r_index, USER_OBJECT_ r_addFinalizer)
 {
-    xmlNodePtr node, parent, ptr;
+    xmlNodePtr node, ptr;
     int i = 0, idx;
     node = (xmlNodePtr) R_ExternalPtrAddr(r_node);    
     ptr = node->children;
@@ -1735,14 +1717,13 @@ R_getChildByIndex(USER_OBJECT_ r_node, USER_OBJECT_ r_index, USER_OBJECT_ r_addF
 USER_OBJECT_
 R_getChildByName(USER_OBJECT_ r_node, USER_OBJECT_ r_index, USER_OBJECT_ r_addFinalizer)
 {
-    xmlNodePtr node, parent, ptr;
-    int i = 0, idx;
+    xmlNodePtr node, ptr;
     node = (xmlNodePtr) R_ExternalPtrAddr(r_node);    
     ptr = node->children;
     const char *name = CHAR_DEREF(STRING_ELT(r_index, 0));
 
     while(ptr) {
-	if(ptr->name && strcmp(name, ptr->name) == 0)
+	if(ptr->name && strcmp(name, (const char *)ptr->name) == 0)
 	    break;
 	ptr = ptr->next;
     }
@@ -1787,14 +1768,14 @@ R_childStringValues(SEXP r_node, SEXP r_len, SEXP r_asVector, SEXP r_encoding, S
 
     for(i = 0, kid = node->children; kid && i < len; i++, kid = kid->next) {
 	tmp  = xmlNodeGetContent(kid);	
-	SEXP val = mkCharCE(tmp, encoding);
+	SEXP val = mkCharCE((const char *)tmp, encoding);
 	PROTECT(val);
 	if(asVector)
 	    SET_STRING_ELT(ans, i, val);
 	else
 	    SET_VECTOR_ELT(ans, i, ScalarString(val));
 	if(names && kid->name) {
-	    SET_STRING_ELT(names, i, mkCharCE(kid->name, encoding));
+	    SET_STRING_ELT(names, i, mkCharCE((const char *)kid->name, encoding));
 	}
 	UNPROTECT(1);
     }
@@ -1805,3 +1786,33 @@ R_childStringValues(SEXP r_node, SEXP r_len, SEXP r_asVector, SEXP r_encoding, S
     UNPROTECT(nprotect);
     return(ans);
 }
+
+
+
+USER_OBJECT_
+R_replaceNodeWithChildren(USER_OBJECT_ r_node)
+{
+    xmlNodePtr node = (xmlNodePtr) R_ExternalPtrAddr(r_node);    
+
+    xmlNodePtr nxt = node->next;
+
+    if(node->prev) {
+	node->prev->next = node->children;
+	node->children->prev = node->prev;
+    } else if(node->parent)
+	node->parent->children = node->children;
+    
+    if(node->children) {
+	xmlNodePtr cur = node->children;
+	while(cur->next) {
+	    cur->parent = node->parent;
+	    cur = cur->next;
+	}
+
+	cur->next = nxt;
+	if(nxt)
+	    nxt->prev = cur;
+    }
+
+    return(NULL_USER_OBJECT);
+}
